// REEF BLOWER

local PULLRANGE = 256*FRACUNIT
local PULLHEIGHT = 64*FRACUNIT

// P_AddBlowerItem
//
// Adds an item to the reef blower
//
local function P_AddBlowerItem(sponge, item)
	
	local mobj = {}
	local things = sponge.ssvars.ammo and sponge.ssvars.ammo["things"] and sponge.ssvars.ammo["things"][SS_POWERUP_BLOWER] or {}
	local oldtype = item.extravalue3
	local maxammo = supersponge_powerup_info[SS_POWERUP_BLOWER].maxammo

	// Already full?!
	if (things != {} and #things >= maxammo) then CONS_Printf(sponge, "Your reef blower is full!") return end	
	
	// Reset item type back to default if not already.
	if (item.type == MT_SUPERSPONGE_FAKEMISSILE and oldtype) then
		item.type = oldtype
	end
	
	mobj = {
		type = item.type,
		sprite = item.sprite,
		flags = item.flags,
		color = item.color,
		colorized = item.colorized,
		scale = item.scale,
		deathstate = item.info.deathstate
	}
	
	table.insert(sponge.ssvars.ammo.things[SS_POWERUP_BLOWER], mobj)	
end

// P_RemoveBlowerItem
//
// Removes an item from the reef blower
//
local function P_RemoveBlowerItem(sponge)

	local things = sponge.ssvars.ammo and sponge.ssvars.ammo["things"] and sponge.ssvars.ammo["things"][SS_POWERUP_BLOWER] or {}
	
	// Nothing inside.
	if (things == {}) then CONS_Printf(sponge, "Your reef blower is empty!") return end
	
	for i = supersponge_powerup_info[SS_POWERUP_BLOWER].maxammo, #things, -1 do
		// Found a valid item.
		if (things[i] != nil) then
			things[i] = nil
			break
		end
	end
end

// P_GetLastBlowerItem
//
// Returns the last item from the reef blower.
//
local function P_GetLastBlowerItem(sponge)

	local things = sponge.ssvars.ammo and sponge.ssvars.ammo["things"] and sponge.ssvars.ammo["things"][SS_POWERUP_BLOWER] or {}
	
	// Nothing inside.
	if (things == {}) then CONS_Printf(sponge, "Your reef blower is empty!") return nil end
	
	for i = supersponge_powerup_info[SS_POWERUP_BLOWER].maxammo, #things, -1 do
		
		if (things[i] == nil) then
			continue
		end
		
		return things[i] // Found an item.
	end
	
	CONS_Printf("Could not find any valid items...")
	
	return nil
end

// P_GetBlowerCount
//
// Returns the count of items from a Reef Blower.
//
rawset(_G, "P_GetBlowerCount", function(sponge)
	local count = 0
	local things = sponge.ssvars.ammo and sponge.ssvars.ammo["things"] and sponge.ssvars.ammo["things"][SS_POWERUP_BLOWER] or {}
	for i = 1, supersponge_powerup_info[SS_POWERUP_BLOWER].maxammo, 1 do
		if (things[i] != nil) then
			count = $ + 1
		end
	end
	return count
end)

local function P_SetVacuumStatus(sponge, status)
	sponge.ssvars.blower.status = status
end	

local function P_GetVacuumStatus(sponge)
	return sponge.ssvars.blower.status
end	

rawset(_G, "P_RemoveBlower", function(sponge)

	local blower = sponge.ssvars.blower
	local obstructor = sponge.ssvars.blower.obstructor or nil
	local cooldown = 10
	
	if (obstructor and obstructor.valid) then
	
		obstructor.rbplayer = sponge.mo // Get the last Spongebob who previously launched you.
		obstructor.rbangle = sponge.drawangle // Get the last angle you were launched at.
		obstructor.rblaunchtimer = 8 // Set pain time
		obstructor.rbflags = RBF_LAUNCHED // "Launch" whatever is in the way.	
		
		P_MoveOrigin(obstructor, sponge.mo.x + P_ReturnThrustX(sponge.mo, sponge.drawangle, -obstructor.oldradius), 
							     sponge.mo.y + P_ReturnThrustY(sponge.mo, sponge.drawangle, -obstructor.oldradius), 
								 sponge.mo.z)
		
		P_InstaThrust(obstructor, sponge.drawangle, FRACUNIT)
		
		obstructor.height = obstructor.oldheight // Restore previous height
		obstructor.radius = obstructor.oldradius // Restore previous radius
	
		cooldown = TICRATE // After launching your obstructor, the blower will be unsuable for one second.
	end

	sponge.ssvars.blower.obstructor = nil
	sponge.ssvars.blower.status = RB_STATUS_OFF
	sponge.ssvars.blower.cooldown = cooldown
	
	// Remove the blower hitbox if it still exists.
	if (blower.hitbox and blower.hitbox.valid) then
		P_RemoveMobj(blower.hitbox)
	end
	
	// If the reef blower was making noise, emit the blower stop sound.
	if (sponge.ssvars.blower.noise) then	
		S_StartSound(sponge.mo, sfx_rbstop)
	end
	
	sponge.ssvars.blower.noise = 0
end)

// Same thing but with explosions!
local function P_ExplodeBlower(sponge)

	local blower = sponge.ssvars.blower
	local obstructor = sponge.ssvars.blower.obstructor or nil
	local dust = P_SpawnMobjFromMobj(sponge.mo, 0, 0, 0, MT_TNTDUST)
	local bomb = P_SpawnMobjFromMobj(sponge.mo, 0, 0, 0, MT_RAY)
	
	// The blower is now gone, so remove it immediately.
	P_RemoveSSPowerup(sponge, SS_POWERUP_BLOWER, false)
	
	sponge.ssvars.blower.obstructor = nil
	sponge.ssvars.blower.status = RB_STATUS_OFF
	sponge.ssvars.blower.cooldown = cooldown
	
	// Removethe blower hitbox if it still exists.
	if (blower.hitbox and blower.hitbox.valid) then
		P_RemoveMobj(blower.hitbox)
	end
	
	dust.destscale = $*4
	
	S_StartSound(bomb.subsector.sector, sfx_bebomb) // Cue the explosion noise.
	
	// First, kill the obstructor.
	if (obstructor and obstructor.valid) then
		P_DamageMobj(obstructor, nil, nil, 0, DMG_INSTAKILL)
	end	
	
	// Now, kill the player.
	if (sponge.mo and sponge.mo.valid) then
		P_DamageMobj(sponge.mo, nil, nil, 0, DMG_INSTAKILL)
	end
	
	P_RadiusAttack(bomb, sponge.mo, 512*FRACUNIT, 100, true)
end

rawset(_G, "P_LaunchVacuumItem", function(sponge)

	local item
	local item_type
	local rocket
	local obstructor

	// Something is blocking the reef blower
	if (P_GetVacuumStatus(sponge) == RB_STATUS_STUCK) then
	
		local aiming = sponge.cmd.aiming << 16
	
		sponge.weapondelay = 1
	
		obstructor = sponge.ssvars.blower.obstructor

		P_RemoveBlower(sponge) // Remove the blower before launching the obstructor
		
		if not (obstructor and obstructor.valid and obstructor.health > 0) then
			return // already dead or non-existent
		end
	
		obstructor.rbplayer = sponge.mo // Get the last Spongebob who previously launched you.
		obstructor.rbangle = sponge.drawangle // Get the last angle you were launched at.
		obstructor.rblaunchtimer = 8 // Set pain time
		obstructor.rbflags = RBF_LAUNCHED // "Launch" whatever is in the way.
		
		P_MoveOrigin(obstructor, sponge.mo.x + P_ReturnThrustX(sponge.mo, sponge.drawangle, -obstructor.oldradius), 
							     sponge.mo.y + P_ReturnThrustY(sponge.mo, sponge.drawangle, -obstructor.oldradius), 
								 sponge.mo.z)
		
		obstructor.height = obstructor.oldheight
		obstructor.radius = obstructor.oldradius
								 
		P_InstaThrust(obstructor, sponge.drawangle, 32*FRACUNIT) // Thrust 32*FRACUNITS forwards.
		P_SetObjectMomZ(obstructor, P_ReturnThrustY(obstructor, aiming, 24*FRACUNIT), true) // Thrust 32*FRACUNITS vertically.

		sponge.ssvars.blower.cooldown = SS_SHOOT_COOLDOWN
		
	// Fire any stored objects if the reef blower is not empty
	elseif (P_GetBlowerCount(sponge) > 0) then
	
		sponge.weapondelay = 1
		sponge.drawangle = sponge.mo.angle
		
		item = P_GetLastBlowerItem(sponge)
		
		rocket = P_SpawnSpongeMissile(sponge, item, false)

		// Boost X / Y momentum
		rocket.momx = FixedMul(40*FRACUNIT, cos(sponge.mo.angle))
		rocket.momy = FixedMul(40*FRACUNIT, sin(sponge.mo.angle))
		
		// Adjust to aiming view
		rocket.momx = FixedMul($, cos(sponge.aiming))
		rocket.momy = FixedMul($, cos(sponge.aiming))
		
		// Boost Z momentum
		rocket.momz = FixedMul(39*FRACUNIT, sin(sponge.aiming))
		
		// Try multiplying momentum with scale
		rocket.momx = FixedMul($, rocket.scale)
		rocket.momy = FixedMul($, rocket.scale)
		rocket.momz = FixedMul($, rocket.scale)
		
		P_RemoveBlowerItem(sponge)
		
		sponge.ssvars.blower.cooldown = SS_SHOOT_COOLDOWN/2
	end
end)

// P_UseReefBlower
//
// Handles the usage of the reef blower.
//
rawset(_G, "P_UseReefBlower", function(sponge)

	local ang
	local x, y, z
	local hitbox
	
	ang = sponge.drawangle
	
	x = sponge.mo.x * cos(ang)
	y = sponge.mo.y * sin(ang)
	z = sponge.mo.z
	
	// Spawn blower hitbox, if it doesn't already exist
	if not (sponge.ssvars.blower.hitbox and sponge.ssvars.blower.hitbox.valid) then
		sponge.ssvars.blower.hitbox = P_SpawnMobjFromMobj(sponge.mo, 0, 0, 0, MT_SUPERSPONGE_VACBOX)	
		sponge.ssvars.blower.hitbox.target = sponge.mo
	end
	
	sponge.ssvars.blower.status = RB_STATUS_ON // on
end)

rawset(_G, "P_IsMobjVacuumable", function(mobj)

	local suck = false
	local whitelist = supersponge_blower_whitelist
	local is_collectible = false
	local is_valid_target = false
	local in_whitelist = false
	
	is_valid_target = (mobj.flags & (MF_ENEMY|MF_MONITOR|MF_MISSILE)) or false
	in_whitelist = whitelist[mobj.type] != nil and whitelist[mobj.type] >= 1 or false
	is_collectible = supersponge_collectible_list[mobj.type] != nil and supersponge_collectible_list[mobj.type] or false
	
	// NEVER and I mean NEVER target shockwaves.
	if (mobj.type == MT_SHOCKWAVE) then
		is_valid_target = false
	end
	
	// Just go with the assumption that golden monitors are extremely heavy.
	if (mobj.flags & MF_MONITOR and mobj.is_golden) then
		is_valid_target = false
	end
	
	// Only suck enemies, monitors, and missiles or whitelisted objects.
	if (mobj and mobj.valid and mobj.health > 0 and (is_valid_target or in_whitelist or is_collectible)) then
		suck = true
	end
	
	return suck
end)

local function P_SSVacuumCheck(player, target)

	local whitelist = supersponge_blower_whitelist

	// The target does not obstruct the blower, so it's okay to vacuum.
	if not (whitelist[target.type] and whitelist[target.type] == 2) then
		return true
	end

	player.ssvars.blower.obstructor = target // set obstructor
	player.ssvars.blower.obstructor.oldheight = target.height
	player.ssvars.blower.obstructor.oldradius = target.radius
	player.ssvars.blower.obstructor.rbflags = RBF_OBSTRUCT
	
	// For enemies, set current state to spawnstate and set reactiontime.
	if (target.flags & MF_ENEMY) then
		target.state = target.info.spawnstate
		target.reactiontime = TICRATE
	end

	// Set obstructor's height and radius
	if (player.ssvars.blower.mo and player.ssvars.blower.mo.valid) then
		target.height = player.ssvars.blower.mo.height
		target.radius = player.ssvars.blower.mo.radius
	end
	
	P_SetVacuumStatus(player, RB_STATUS_STUCK) // set status
	
	return false
end

local function P_DoBlowerPop(blower, monitor)

	P_KillMobj(monitor, blower, blower.target)
	
	monitor.state = S_BOX_POP2 // Set state to S_BOX_POP2
	monitor.sprite = SPR_NULL // Hide monitor sprite.
	
	// If the monitor can respawn outside of Singleplayer / Coop, move it back to it's origin for the next monitor.
	if not G_CoopGametype() then
		P_SetOrigin(monitor, monitor.origin.x, monitor.origin.y, monitor.origin.z)
	end
	
	monitor.momx = 0
	monitor.momy = 0
	monitor.momz = 0
end

// P_SSVacuumCapture
//
// Special items handler for the Reef Blower
//
rawset(_G, "P_SSVacuumCapture", function(blower, target)

	local type
	local sponge
	
	type = target.type
	sponge = blower.target.player
	
	// No valid player...
	if not (sponge and sponge.valid) then
		return false
	end
	
	// Check if the reef blower is full before adding!
	if (P_GetBlowerCount(sponge) >= supersponge_powerup_info[SS_POWERUP_BLOWER].maxammo) then
		return false
	end
	
	// If our target was the Egg Shield, break it.
	if (target.type == MT_EGGSHIELD) then
	
		P_KillMobj(target, blower.target, blower.target, 0) 
		
		S_StartSound(target, sfx_wbreak)
		
		return false 
	end
	
	// Spawn collectible from blower's source.
	if P_IsMobjCollectible(target) then P_SpawnMobjFromMobj(blower.target, 0, 0, 0, type) end	
	
	// Kill monitors.
	if (target.flags & MF_MONITOR) then P_DoBlowerPop(blower, target) return true end
	
	// Special handling for big targets!
	if not P_SSVacuumCheck(sponge, target) then
		return false
	end
	
	// Add enemy / missile to the reef blower list
	if (target.flags & (MF_ENEMY|MF_MISSILE) or target.type == MT_ENERGYBALL) then
		P_AddBlowerItem(blower.target.player, target)
	end
	
	if (target.type == MT_PYREFLY) then
		P_RemovePyreFly(target) // Specialized removal for Pyreflies
	elseif (target.type == MT_GSNAPPER or target.type == MT_CRUSHSTACEAN or target.type == MT_BANPYURA) then
		P_RemoveSegmentedEnemy(target) // Specialized removal for multi-segmented enemies.
	else
		P_RemoveMobj(target)
	end
end)

// Otherwise known as...not P_Telekinesis!
local function P_SSVacuumPull(blower, speed, target)
	
	local an = R_PointToAngle2(blower.x, blower.y, target.x, target.y)
	local dist = P_AproxDistance(P_AproxDistance(blower.x - target.x, blower.y - target.y), blower.z - target.z)
	local ignore = false
	
	if (dist < 1) then
		dist = 1
	end
	
	speed = FixedMul(speed, speed/FixedInt(dist)) + 1
	
	if (target and target.player and dist > MELEERANGE and P_GetPlayerControlDirection(target.player) == 2) then
		if (target.player.pflags & PF_JUMPED)
		or (target.player.pflags & (PF_SPINDOWN|PF_SPINNING)) then
			speed = $ * -1
		end
	end
	
	P_Thrust(target, an, -speed)
	
	// If your target is way above you or below you, try pulling them towards your height.
	if (blower.z > target.z + target.height) then
		P_SetObjectMomZ(target, speed, true)
	elseif (blower.z < target.z - target.height) then
		P_SetObjectMomZ(target, -speed, true)
	end

	if (target.type == MT_GOLDBUZZ or target.type == MT_REDBUZZ) then
		target.tics = $ + 8
	end
end

local function SpongeVacuumSearch(blower, target)

	local sponge
	local span
	local height
	local dist
	local angle
	local pullrange
	local pullheight

	span = ANGLE_90
	sponge = blower.target.player
	
	if not (target and target.valid) then return end // Target does not exist
	
	if (target == blower or target == blower.target) return end // Don't target your owner or yourself!
	
	if not P_IsMobjVacuumable(target) then return end // Can't be vacuumed...
	
	if (target.target == blower.target and (target.flags & MF_MISSILE) and gametyperules & GTR_RINGSLINGER) then return end // In ringslinger, do not vacuum your own projectiles!
	
	pullrange = PULLRANGE
	pullheight = PULLHEIGHT
	
	// Check for other players who may be present in the vicinity.
	if (target and target.player and target.player.valid) then
		if (P_IsSuperSponge(target.skin) and target.player.ssvars.powerups[SS_POWERUP_BLOWER]) then
			return // DO NOT pull in players who are currently using a Reef Blower.
		end
		if (target.player.powers[pw_super]) then
			pullrange = $/2  // The pull range for super players is reduced by half.
		end
	end

	height = abs(target.z - blower.target.z)
	dist = P_AproxDistance(target.x - blower.target.x, target.y - blower.target.y)
	angle = R_PointToAngle2(blower.x, blower.y, target.x, target.y) - blower.target.player.drawangle
	
	if not (dist <= pullrange) // Not in pulling distance.
	or not (height <= FixedMul(pullheight, blower.scale)) // The height distance is greater than the pull height.
	or not (angle < ANGLE_90 and angle > ANGLE_270) // The target is behind us.
	or not (P_CheckSight(blower, target)) then // can't see the target
		
		if (target.sucker == blower) then
			target.sucker = nil 
		end	
		
		if (P_IsMobjCollectible(target) and (target.flags & MF_NOGRAVITY)) then
			target.momx = 0
			target.momy = 0
			target.momz = 0
		end

		return
	end
	
	// Reel' em in!
	if (target.sucker != blower)
		target.sucker = blower
	end
	
	local homedist = P_AproxDistance(P_AproxDistance(target.x - blower.x, target.y - blower.y), target.z - blower.z) // Homing distance
	local speed = (target.type == MT_ENERGYBALL and 40*FRACUNIT) or 8*FRACUNIT // Speed
	
	 // Continuously pull the target towards the blower if not close.
	if (homedist > blower.height*2) then 
		P_SSVacuumPull(blower, speed, target)
	else
		P_SSVacuumCapture(blower, target)
	end
end

// P_SpongeVacuumThink
//
// Handles the search function for the Reef Blower. 
// Some code from P_LookForEnemies is used as reference.
//
rawset(_G, "P_SpongeVacuumThink", function(blower)
	
	local x1, x2, y1, y2
	
	// The reef blower's broken!
	if not (blower and blower.valid) then
		return
	end

	// Who's using the reef blower?
	if not (blower.target and blower.target.valid) then
		return
	end
	
	x1 = blower.x - PULLRANGE/2
	x2 = blower.x + PULLRANGE/2
	y1 = blower.y - PULLRANGE/2
	y2 = blower.y + PULLRANGE/2

	local blockmap = searchBlockmap("objects", SpongeVacuumSearch, blower, x1, x2, y1, y2) // Find other nearby objects and pull them in.
end)

//
// MobjBlowerCheck
//
// Mobj Handler for Blower
//
local function MobjBlowerCheck(mobj)
	
	local sponge
	local height
	local dist
	local blower
	local monitor

	if not P_IsMobjVacuumable(mobj) then return end // ignore non-vacuumable objects
	
	// This timer acts a short interval before objects are allowed to hurt players while being launched.
	if (mobj.rblaunchtimer) then
	
		mobj.rblaunchtimer = $ - 1
		
		// Remove MF_NOCLIPTHING for MT_SPARRINGDUMMY.
		if (BattleModIsActive() and mobj.rblaunchtimer == 0 and mobj.type == MT_SPARRINGDUMMY) then
			mobj.flags = $ & ~MF_NOCLIPTHING
		end
	end
	
	// Obstructing a Reef Blower
	if (mobj.rbflags == RBF_OBSTRUCT and mobj.sucker and mobj.sucker.valid and mobj.sucker.target and mobj.sucker.target.valid) then
		
		local blower = mobj.sucker
		local player = mobj.sucker.target.player
		local angle = player and player.drawangle
		
		// We don't want enemies to hurt the player while obstructing the blower.
		if (mobj.flags & MF_ENEMY) then
			mobj.reactiontime = TICRATE
		end

		// Special handling for the BattleMod "Sparring Dummy"
		if (BattleModIsActive() and mobj.type == MT_SPARRINGDUMMY) then
			mobj.rbdoll = true
			mobj.pushed = false
			mobj.pushed_last = nil
			mobj.pushtics = 0
		end
		
		// For players, send them into a "rolling" state while obstructing the blower.
		if (mobj.player) then
		
			local player_can_spin = mobj.player.charflags and not (mobj.player.charflags & SF_NOJUMPSPIN)
				
			// If the player can perform a spin attack ala Sonic, act like a "mini-chainsaw"
			if (player_can_spin) then
				mobj.state = S_PLAY_ROLL
				mobj.player.pflags = $|PF_SPINNING
			else
				mobj.state = S_PLAY_FALL
			end
			
			mobj.player.powers[pw_nocontrol] = 1
		end

		// Set the blower's position in front of the pack's "nozzle"
		P_MoveOrigin(mobj, blower.target.x, blower.target.y, blower.target.z + FRACUNIT)
		P_InstaThrust(mobj, angle, 32*FRACUNIT)
		P_Thrust(mobj, angle-ANGLE_90, 16*FRACUNIT)
		
		// Reset Z momentum to keep mobj above ground.
		mobj.momz = 0
		
		return
	end
	
	// Being launched in mid air.
	if (mobj.rbflags == RBF_LAUNCHED) then
	
		local spongebob = mobj.rbplayer and mobj.rbplayer.valid or nil
		local blocked = P_TryMove(mobj, mobj.x + mobj.momx, mobj.y + mobj.momy, true) != true
		local speed = P_AproxDistance(mobj.momx, mobj.momy)
		local stop = false
		
		P_Thrust(mobj, mobj.rbangle, FRACUNIT)
		
		// Special handling for the BattleMod "Sparring Dummy"
		if (mobj.type == MT_SPARRINGDUMMY) then
			mobj.flags = $ & ~MF_NOCLIPTHING
		end
		
		// Hit the ground.
		if P_IsObjectOnGround(mobj) then
			stop = true
		end
		
		// In pain.
		if (mobj and mobj.player and P_PlayerInPain(mobj.player)) then
			stop = true
		end
		
		// We must have hit a wall or something
		if (blocked and (mobj.flags & MF_SHOOTABLE) and mobj.type != MT_PLAYER) then	
		
			P_DamageMobj(mobj, spongebob, spongebob, 1)
		
			mobj.rbflags = $ & ~RBF_LAUNCHED // we hit a wall, so stop.
			
			return
		end
		
		// For players, set their state to a semi-invincible rolling state to mimic being launched.
		if (mobj.player and mobj.player.valid) then
			
			mobj.player.pflags = $|PF_JUMPED
			
			if (mobj.state != S_PLAY_ROLL) then
				mobj.state = S_PLAY_ROLL
			end
		end
		
		// If we hit the ground or got damaged while being launched, remove the flag.
		if (stop) then
		
			// For players, remove the jump flag and put them into the falling state.
			if (mobj.player) then
				mobj.state = S_PLAY_FALL
				mobj.player.pflags = $ & ~PF_JUMPED
			end
		
			mobj.rbflags = $ & ~RBF_LAUNCHED
		end
		
		return
	end
	
	// The mobj is no longer alive or the reef blower is no longer present
	if not (mobj and mobj.valid and mobj.health > 0 and mobj.sucker and not mobj.sucker.valid) then return end
	
	// Reset momentum for floating / non-moving collectibles.
	if (supersponge_collectible_list[mobj.type] and supersponge_collectible_list[mobj.type] != 2)
		mobj.momx = 0
		mobj.momy = 0
		mobj.momz = 0
	end
	
	mobj.sucker = nil
end
addHook("MobjThinker", MobjBlowerCheck)

local function BlowerThinker(blower)

	local obstructor

	// Who's running the blower?
	if not (blower and blower.valid and blower.target and blower.target.valid and blower.target.player) then
		return
	end
	
	local sponge = blower.target.player
	local angle = sponge.drawangle
	local ammo = P_GetBlowerCount(sponge)
	local maxammo = supersponge_powerup_info[SS_POWERUP_BLOWER].maxammo
	
	// If the blower is full or the player is currently super, stop it from running!
	if (ammo == maxammo or sponge.powers[pw_super] > 0)
		return P_RemoveBlower(sponge)
	end
	
	obstructor = sponge.ssvars.blower.obstructor
	
	// The blower will explode after 30 seconds in non-cooperative gametypes.
	if not G_CoopGametype() then
	
		local timer = blower.cusval
	
		// If something is obstructing the reef blower, add to the explosion timer.
		if (obstructor and obstructor.valid and obstructor.rbflags == RBF_OBSTRUCT) then
			blower.cusval = $ + 1
		else
			blower.cusval = 0
		end
		
		if (timer >= 30*TICRATE) then P_ExplodeBlower(sponge) return end // KABOOM!
	
		// Uh oh, looks like the blower is about to blow!
		if (timer >= 15*TICRATE) then
		
			local x,y
			local smoke
					
			x = P_ReturnThrustX(nil, angle, -sponge.mo.radius*2)
			y = P_ReturnThrustY(nil, angle, -sponge.mo.radius*2)
			
			// In the last twenty seconds, notify the player that it is about to explode.
			if (timer == 20*TICRATE) then
				S_StartSound(blower, sfx_s3kd4s)
			end
			
			// Every 8 seconds, spawn smoke particles behind the player.
			if (timer % 8 == 0) then
				smoke = P_SpawnMobjFromMobj(sponge.mo, x, y, sponge.mo.height/2, MT_SMOKE)
				smoke.scale = P_RandomRange(1/2, 2)*FRACUNIT
				smoke.momz = 5*FRACUNIT
			end
		end
	end
	
	// Something is blocking the Reef Blower!
	if (obstructor and obstructor.valid and obstructor.rbflags != RBF_OBSTRUCT)	
		sponge.ssvars.blower.obstructor.rbflags = RBF_OBSTRUCT // Set the "obstruct" flag, if our obstructor does not currently have one.
	end
	
	// Do not move the nozzle, if our blower no longer exists.
	if not (blower and blower.valid) then
		return
	end
	
	// Set the blower's position in front of the pack's "nozzle"
	P_MoveOrigin(blower, blower.target.x, blower.target.y, blower.target.z)
	P_InstaThrust(blower, angle, 32*FRACUNIT)
	P_Thrust(blower, angle-ANGLE_90, 16*FRACUNIT)
end
addHook("MobjThinker", BlowerThinker, MT_SUPERSPONGE_VACBOX)