// Animation Constants
rawset(_G, "SS_WPANIM_NONE", 0)
rawset(_G, "SS_WPANIM_BOB", 1)
rawset(_G, "SS_WPANIM_PULSATE", 2)

// Sprite Constants
local SPR_INDEX_NONE = 0
local SPR_INDEX_LAUNCHER = 1
local SPR_INDEX_NET = 2
local SPR_INDEX_WAND = 3
local SPR_INDEX_ALTFIRE = 4
local SPR_INDEX_BLOWER = 5

local function GetVacuumSize(count)

	local scale

	if (count >= SS_BLOWER_LIMIT) then
		scale = 14*FRACUNIT/10 // 1.4
	elseif (count >= 5 and count <= SS_BLOWER_LIMIT) then
		scale = 12*FRACUNIT/10 // 1.2
	else
		scale = FRACUNIT
	end
	
	return scale
end

local function WP_WeaponThink(player, weapon)

	local blower_ammo = P_GetBlowerCount(player)
	local net_ammo = P_GetNetCount(player)
	local launcher_ammo = player.ssvars.ammo["jellyfish"]
	local holding_spin = player.ssvars.spinbtntimer >= 2
	local scale = player.ssvars.wp_sprites[weapon].scale
	local maxscale = player.ssvars.wp_sprites[weapon].maxscale
	local basescale = player.ssvars.wp_sprites[weapon].basescale

	if (weapon == SS_POWERUP_BLOWER and blower_ammo) then
		player.ssvars.wp_sprites[weapon].scale = GetVacuumSize(blower_ammo) // The scale changes depending on count.
	elseif (weapon == SS_POWERUP_NET and net_ammo) then
		player.ssvars.wp_sprites[weapon].anim = SS_WPANIM_BOB // Set to bobbing animation.
	elseif (weapon == SS_POWERUP_LAUNCHER and holding_spin and launcher_ammo) then
		player.ssvars.wp_sprites[weapon].anim = SS_WPANIM_PULSATE // Set to pulsate animation.
	end
end

local function WP_WeaponBob(player, weapon)
	
	local ammo = 0
	local ammotype = supersponge_powerup_info[weapon].ammotype or "None"
	local scale = player.ssvars.wp_sprites[weapon].scale
	local maxscale = player.ssvars.wp_sprites[weapon].maxscale
	local basescale = player.ssvars.wp_sprites[weapon].basescale
	local var = player.ssvars.wp_sprites[weapon].var or 0
	
	// Get ammo based on the current weapon.
	if (weapon == SS_POWERUP_BLOWER) then
		ammo = P_GetBlowerCount(player)
	elseif (weapon == SS_POWERUP_NET) then
		ammo = P_GetNetCount(player)
	else
		ammo = player.ssvars.ammo[ammotype] or 0
	end
	
	// If we ran out of ammo, reset the current animation.
	if not (ammo) then	
		player.ssvars.wp_sprites[weapon].scale = FRACUNIT // Reset scale.
		player.ssvars.wp_sprites[weapon].anim = SS_WPANIM_NONE // Reset animation.
		player.ssvars.wp_sprites[weapon].var = 0 // Reset extra value.
		player.ssvars.wp_tics = 0 // Reset tics.	
	else
		// "Bob" the weapon by alternating between shrinking and growing.
		if (var) then
			player.ssvars.wp_sprites[weapon].scale = $ - FRACUNIT/32 // Shrink
			if (player.ssvars.wp_sprites[weapon].scale <= basescale) then
				player.ssvars.wp_sprites[weapon].var = 0
			end
		else
			player.ssvars.wp_sprites[weapon].scale = $ + FRACUNIT/32 // Grow
			if (player.ssvars.wp_sprites[weapon].scale >= maxscale) then
				player.ssvars.wp_sprites[weapon].var = 1
			end
		end
	end
end

local function WP_WeaponPulsate(player, weapon)
	
	local holding_spin = player.ssvars.spinbtntimer
	local scale = player.ssvars.wp_sprites[weapon].scale
	local maxscale = player.ssvars.wp_sprites[weapon].maxscale
	local basescale = player.ssvars.wp_sprites[weapon].basescale
	local var = player.ssvars.wp_sprites[weapon].var or 0
	local angle = player.ssvars.wp_sprites[weapon].angle or 0
	
	// Scale the weapon up until it reaches max size.
	if (holding_spin) 
		if (scale < maxscale) then
			player.ssvars.wp_sprites[weapon].scale = $ + FRACUNIT/128
			if (player.ssvars.wp_sprites[weapon].scale > maxscale) then
				player.ssvars.wp_sprites[weapon].scale = maxscale
			end
		elseif (var == 0) then
			player.ssvars.wp_sprites[weapon].var2 = $ + FRACUNIT/64
			if (player.ssvars.wp_sprites[weapon].var2 >= FRACUNIT/48) then
				player.ssvars.wp_sprites[weapon].var = 1
			end
		elseif (var == 1) then
			player.ssvars.wp_sprites[weapon].var2 = $ - FRACUNIT/64
			if (player.ssvars.wp_sprites[weapon].var2 <= 0) then
				player.ssvars.wp_sprites[weapon].var = 0
			end		
		end
	else
		player.ssvars.wp_sprites[weapon].scale = basescale
		player.ssvars.wp_sprites[weapon].tics = 1
		player.ssvars.wp_sprites[weapon].angle = 0
		player.ssvars.wp_sprites[weapon].var = 0
		player.ssvars.wp_sprites[weapon].var2 = 0
		player.ssvars.wp_sprites[weapon].anim = SS_WPANIM_NONE
	end
end

// Weapon Sprite Table
rawset(_G, "supersponge_weapon_sprites", {
	[SS_POWERUP_BLOWER] = {
			anim = SS_WPANIM_NONE,
			scale = FRACUNIT,
			basescale = FRACUNIT,
			maxscale = 2*FRACUNIT,
			var1 = 0,
			var2 = 0,
			angle = 0,
		},
		[SS_POWERUP_NET] = {
			anim = SS_WPANIM_NONE,
			scale = FRACUNIT,
			basescale = FRACUNIT,
			maxscale = 12*FRACUNIT/10,
			var1 = 0,
			var2 = 0,
			angle = 0
		},
		[SS_POWERUP_LAUNCHER] = {
			anim = SS_WPANIM_NONE,
			scale = FRACUNIT,
			basescale = FRACUNIT,
			maxscale = 11*FRACUNIT/10,
			var1 = 0,
			var2 = 0,
			angle = 0
		},
		[SS_POWERUP_WAND] = {
			anim = SS_WPANIM_NONE,
			scale = FRACUNIT,
			basescale = FRACUNIT,
			maxscale = 2*FRACUNIT,
			var1 = 0,
			var2 = 0,
			angle = 0
		}
})

// Weapon Frame Table
local supersponge_weapon_frames = {
	[SS_POWERUP_BALLOON] = A,
	[SS_POWERUP_BLOWER] = B,
	[SS_POWERUP_NET] = C,
	[SS_POWERUP_LAUNCHER] = D,
	[SS_POWERUP_WAND] = E
}

// Weapon Animation Table
local supersponge_weapon_animations = {
	[SS_WPANIM_NONE] = {
		tics = 1,
		action = WP_WeaponThink,
		next = SS_WPANIM_NONE
	},
	[SS_WPANIM_BOB] = {
		tics = 1,
		action = WP_WeaponBob,
		next = SS_WPANIM_BOB
	},
	[SS_WPANIM_PULSATE] = {
		tics = 1,
		action = WP_WeaponPulsate,
		next = SS_WPANIM_PULSATE
	}
}

local supersponge_handsprites = {
	["idle"] = {
		[SPR_INDEX_NONE] = {A},
		[SPR_INDEX_LAUNCHER] = {F}, // Launcher
		[SPR_INDEX_NET] = {M}, // Net
		[SPR_INDEX_WAND] = {R}, // Bubble
		[SPR_INDEX_ALTFIRE] = {X}, // Alternate Fire
		[SPR_INDEX_BLOWER] = {31} // Blower
	},
	["attack"] = {
		[SPR_INDEX_NONE] = {A, B, C, D, E}, // Melee
		[SPR_INDEX_LAUNCHER] = {F, J, K, L}, // Launcher
		[SPR_INDEX_NET] = {N, O, P, Q}, // Net
		[SPR_INDEX_WAND] = {R, S, T, U, V, W}, // Bubble
		[SPR_INDEX_ALTFIRE] = {X, Y, Z, 26, 27, 28, 29, 30}, // Alternate Fire
		[SPR_INDEX_BLOWER] = {31} // Blower
	}
}

// Y'know, I hate hardcoded functions but I'll make at least one exception for this.
local function DrawWeaponSprite(v, player, weapon)

	local tbl
	local sprite
	local sprite2
	local sprite_name
	local animation
	local index
	local frame
	local colormap
	
	// If RingSlingerNEO is installed, do not show hand sprites during Ringslinger matches.
	if (RingSlinger and gametyperules & GTR_RINGSLINGER) then
		return
	end
	
	tbl = supersponge_handsprites
	sprite2 = player.powers[pw_super] and (player.mo.sprite2 & ~FF_SPR2SUPER) or player.mo.sprite2
	
	// Use the attacking frames, if we're currently moving.
	if (sprite2 == SPR2_MLEE)
	or (sprite2 == SPR2_MLEL)
	or (sprite2 == SPR2_JNET)
	or (sprite2 == SPR2_BWND)
	or (sprite2 == SPR2_FIRE)
	or (sprite2 == SPR2_F2RE)
		animation = "attack"
	else
		animation = "idle"
	end
	
	// Choose weapon sprite based on prefix.
	if (player.powers[pw_super]) then
		sprite_name = "W0P3"
	elseif (player.ssvars.powerups[SS_POWERUP_SUIT]) then
		sprite_name = "W0P2"
	else
		sprite_name = "W0P1"
	end
		
	// Choose index based on the player's current sprite / status.
	if (sprite2 == SPR2_F2RE or player.powers[pw_super]) then
		index = SPR_INDEX_ALTFIRE
	elseif (player.ssvars.powerups[SS_POWERUP_LAUNCHER])
		index = SPR_INDEX_LAUNCHER
	elseif (player.ssvars.powerups[SS_POWERUP_NET]) then
		index = SPR_INDEX_NET
	elseif (player.ssvars.powerups[SS_POWERUP_WAND]) then
		index = SPR_INDEX_WAND
	elseif (player.ssvars.powerups[SS_POWERUP_BLOWER]) then
		index = SPR_INDEX_BLOWER
	else
		index = SPR_INDEX_NONE
	end
	
	local player_frame = (player.mo.frame & FF_FRAMEMASK)
		
	if (animation == "idle") then
		frame = supersponge_handsprites[animation][index][1] // Always use the first frame in table
	elseif (sprite2 == SPR2_MLEL) then
		frame = E // force frame E for SPR2_MLEL
	else
		frame = (tbl[animation][index][player_frame + 1] != nil and tbl[animation][index][player_frame + 1]) or tbl[animation][index][1]
	end
	
	sprite = v.getSpritePatch(sprite_name, frame, 0, 0)
	colormap = v.getColormap(skins[player.skin].name, player.mo.color)
	
	v.drawScaled(0, 0, FRACUNIT, sprite, V_SNAPTORIGHT|V_SNAPTOBOTTOM|V_PERPLAYER, colormap)
end

local function AnimateWeaponIcon(v, x, y, weapon, sprite, player)
	
	local ammo
	local var1
	local var2
	local scale
	local maxscale
	local animation
	local colormap
	local flags
	
	var1 = player.ssvars.wp_sprites[weapon].var1
	var2 = player.ssvars.wp_sprites[weapon].var2
	scale = player.ssvars.wp_sprites[weapon].scale
	maxscale = player.ssvars.wp_sprites[weapon].maxscale
	animation = player.ssvars.wp_sprites[weapon].anim
	colormap = nil
	flags = V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER
	
	// Cycle animations for current weapon.
	local next = supersponge_weapon_animations[animation] and supersponge_weapon_animations[animation].next or SS_WPANIM_NONE
	local next_tics = supersponge_weapon_animations[next] and supersponge_weapon_animations[next].tics or 0
	if (player.ssvars.wp_tics) then
		player.ssvars.wp_tics = $ - 1
	else
		player.ssvars.wp_tics = next_tics
		player.ssvars.wp_sprites[weapon].anim = next
		// Execute function, if it exists.
		if (supersponge_weapon_animations[animation].action != nil) then
			local func = supersponge_weapon_animations[animation].action
			func(player, weapon)
		end
	end
	
	// Set animation / colormap based on the weapon's current state / ammo
	if (weapon == SS_POWERUP_NET) then
		ammo = P_GetNetCount(player)
	elseif (weapon == SS_POWERUP_LAUNCHER) then
		ammo = player.ssvars.ammo["jellyfish"]
	elseif (weapon == SS_POWERUP_WAND) then
		ammo = player.ssvars.ammo["bubbles"]
	end
	
	// Add special effects to the weapon icon depending on its current state or ammo.
	if (weapon == SS_POWERUP_NET and ammo > 0) then
		v.drawScaled(x*FRACUNIT, y*FRACUNIT, scale, v.cachePatch("NETBLOB"), flags, nil) // Draw net blob underneath net.
	elseif (weapon == SS_POWERUP_LAUNCHER and ammo and player.ssvars.spinbtntimer >= SS_LAUNCHER_MAX_TIME) then
		colormap = v.getColormap(TC_DASHMODE, 0) // start fading to redish orange
	end
	
	local counter_x = x + 38 // "X times graphic" x pos
	local counter_y = y + 8 // "X times graphic" y pos
	local counter_num_x = counter_x + 16 // counter num X pos
	local counter_num_y = counter_y - 5 // counter num Y pos
	
	// If the weapon requires ammo, draw the counter.
	if (ammo != nil and supersponge_powerup_info[weapon].ammo) then
		v.draw(counter_x, counter_y, v.cachePatch("STLIVEX"), flags, nil)
		if (ammo >= 10) then
			v.drawNum(counter_num_x + 8, counter_num_y, ammo, flags)
		else
			v.drawNum(counter_num_x, counter_num_y, ammo, flags)		
		end
	end

	v.drawScaled(x*FRACUNIT, y*FRACUNIT, scale, sprite, flags, colormap)
	
	// If the weapon is "pulsating", overlay another weapon sprite on top of the first one.
	if (scale >= maxscale and animation == SS_WPANIM_PULSATE) then
		v.drawScaled(x*FRACUNIT- var2, y*FRACUNIT - var2, scale + var2, sprite, flags, colormap)	
	end
end

local function SB_WeaponHUD(v, player, cam)
	
	local weapon
	local weapon_sprite
	local weapon_frame
	local weapon_angle
	local x, y, x2, y2
	
	if not (player.mo and player.mo.valid and P_IsSuperSponge(player.mo.skin) and CV_FindVar("ss_powerups").value) then return end
	
	weapon = P_GetCurrentSSWeapon(player)
	
	if not (cam.chase) then
		DrawWeaponSprite(v, player, weapon) // Draw the hand sprites for the main HUD.
	end
	
	player.ssvars.camangle = cam.angle
	
	// Not holding any weapons currently
	if (weapon == SS_POWERUP_NONE) then return end
	
	weapon_frame = supersponge_weapon_frames[weapon] or 0
	weapon_angle = player.ssvars.wp_sprites[weapon] and player.ssvars.wp_sprites[weapon].angle or 0
	
	// Use frame F for the Jellyfish Launcher, if we've reached the maximum limit.
	if (weapon == SS_POWERUP_LAUNCHER and player.ssvars.spinbtntimer >= TICRATE and player.ssvars.ammo["jellyfish"]) then
		weapon_frame = F
	end
	
	// If the weapon's sprite frame is in the wp_sprites table, use that, else, use the default sprite.
	if (supersponge_weapon_frames[weapon]) then
		weapon_sprite = v.getSpritePatch(SPR_SSD3, weapon_frame, 0, weapon_angle)
	else
		weapon_sprite = v.getSpritePatch(supersponge_powerup_info[weapon].sprite or SPR_SSDI, supersponge_powerup_info[weapon].frame, 0, weapon_angle)
	end
	
	// "Animate" the weapon sprite.
	if (player.ssvars.wp_sprites[weapon] != nil and weapon_sprite != nil) then
		AnimateWeaponIcon(v, 254, 54, weapon, weapon_sprite, player)
	end
end
customhud.SetupItem("supersponge_hud", "supersponge", SB_WeaponHUD, 0)

