// Spongebob Supersponge Menu Lua
customhud.SetupFont("SBPFN", 0, 4, nil)

local SUPERSPONGE_CREDITS = {"TehRealSalt - Custom HUD Library used in the",
							 "creation of the weapon and menu HUDs.",
							 "Sprkizard - LinePathing.lua used as a base for the",
							 "drawing functions in the mod.",
							 "SMS_Alfredo - Mario Bros. mod used as a reference for", 
							 "the menu, pants color, and pickups.",
							 "Tahutoa - Soundfont and Invincibility Jingle."}
							 
local SUPERSPONGE_CREDITS2 = {"UnwiishedHog - SuperSponge Remixes",
							  "LemurBoy12 - Spongebob's Model",
							  "Sparkette - SRB2 Blender Template",
							  "GianlucaRueger2002 - Clam Sprites",
							  "Nickham13 - Clean rips of the SuperSponge OST"}

// Graphics
local OPT_MENU_BACK = "SS_OPTIONS_BACK"
local OPT_MENU_BORDER = "SS_OPTIONS_BORDER"

// Constants
local DELAY_TIME_BASIC = 8
local DELAY_TIME_FAST = 4
local SPONGEMENU_DEFAULT = 0
local SPONGEMENU_GAMEPLAY = 1
local SPONGEMENU_SECONDCOLOR = 2
local SPONGEMENU_SOUNDS = 3
local SPONGEMENU_CREDITS = 4
local SPONGEMENU_CREDITS2 = 5
local OPT_TYPE_NONE = 0 // Does nothing. Can still press JUMP to run functions.
local OPT_TYPE_TOGGLE = 1 // Displays the specified variable's value to the right of the option.
local OPT_TYPE_MULTI = 2 // A multi-selection option where the variable is displayed to the right with arrows.
local OPT_TYPE_TEXT = 3 // An option which displays text and is non-selectable.

// Options layout

// Each option entry is organized in the following way.
// TYPE - defines what kind of entry this is and how it is displayed:.
// NAME - The entry's name
// FUNC - The entry's function which is executed differently depending on it's type.
// PARAM - The entry's parameter or main value. Each option type will treat this differently as well.
// VARIABLE - The type of variable the entry is targeting. This field can be a cvar or skincolor value.

local OptionList_Default = {
	[1] = {
		type = OPT_TYPE_NONE,
		name = "Gameplay",
		func = "SM_OpenMenu",
		param = SPONGEMENU_GAMEPLAY,
		variable = "none"
	},
	[2] = {
		type = OPT_TYPE_NONE,
		name = "Secondcolor",
		func = "SM_OpenMenu",
		param = SPONGEMENU_SECONDCOLOR,
		variable = "none"
	},
	[3] = {
		type = OPT_TYPE_NONE,
		name = "Sounds",
		func = "SM_OpenMenu",
		param = SPONGEMENU_SOUNDS,
		variable = "none"
	},
	[4] = {
		type = OPT_TYPE_NONE,
		name = "Credits",
		func = "SM_OpenMenu",
		param = SPONGEMENU_CREDITS,
		variable = "none"
	}
}

local OptionList_Gameplay = {
	[1] = {
		type = OPT_TYPE_TOGGLE,
		name = "Toggle Pickups",
		func = "SM_ToggleCvar",
		param = "ss_pickups",
		variable = "cvar"
	},
	[2] = {
		type = OPT_TYPE_TOGGLE,
		name = "Toggle Powerups",
		func = "SM_ToggleCvar",
		param = "ss_powerups",
		variable = "cvar"
	},
	[3] = {
		type = OPT_TYPE_TOGGLE,
		name = "Keep Powerups",
		func = "SM_ToggleCvar",
		param = "ss_keeppowerups",
		variable = "cvar"
	}
}

local OptionList_Secondcolor = {
	[1] = {
		type = OPT_TYPE_MULTI,
		name = "Secondcolor",
		func = nil,
		param = nil,
		items = nil,
		variable = "skincolor"
	}
}

local OptionList_Sounds = {
	[1] = {
		type = OPT_TYPE_TOGGLE,
		name = "Jingles",
		func = "SM_ToggleCvar",
		param = "ss_jingles",
		variable = "cvar"
	},
	[2] = {
		type = OPT_TYPE_TOGGLE,
		name = "Taunts",
		func = "SM_ToggleCvar",
		param = "ss_taunts",
		variable = "cvar"
	},
	[3] = {
		type = OPT_TYPE_TOGGLE,
		name = "Victory Taunts",
		func = "SM_ToggleCvar",
		param = "ss_victorytaunts",
		variable = "cvar"
	},
	[4] = {
		type = OPT_TYPE_TOGGLE,
		name = "Footstep Noises",
		func = "SM_ToggleCvar",
		param = "ss_footsteps",
		variable = "cvar"
	}
}

local OptionList_Credits = {
	[1] = {
		type = OPT_TYPE_TEXT,
		name = "-",
		func = "SM_ExpandCredits",
		param = SUPERSPONGE_CREDITS,
		variable = "none",
		scale = 5*FRACUNIT/10
	}
}
local OptionList_Credits2 = {
	[1] = {
		type = OPT_TYPE_TEXT,
		name = "-",
		func = nil,
		param = SUPERSPONGE_CREDITS2,
		variable = "none",
		scale = 5*FRACUNIT/10
	}
}

// Menus
local SM_DefaultMenu = {
	id = SPONGEMENU_DEFAULT,
	name = "Options Menu",
	options = OptionList_Default,
	cursor_pos = 0,
	spacing = 16,
	row_offset = 0,
	callback = nil
}

local SM_GameplayMenu = {
	id = SPONGEMENU_GAMEPLAY,
	name = "Gameplay Options",
	options = OptionList_Gameplay,
	cursor_pos = 0,
	spacing = 16,
	row_offset = 0,
	callback = "SM_OpenDefaultMenu"
}

local SM_SecondcolorMenu = {
	id = SPONGEMENU_SECONDCOLOR,
	name = "Secondcolor",
	options = OptionList_Secondcolor,
	cursor_pos = 0,
	spacing = 40,
	row_offset = -64,
	callback = "SM_OpenDefaultMenu"
}

local SM_SoundsMenu = {
	id = SPONGEMENU_SOUNDS,
	name = "Sound Options",
	options = OptionList_Sounds,
	cursor_pos = 0,
	spacing = 16,
	row_offset = 0,
	callback = "SM_OpenDefaultMenu"
}

local SM_CreditsMenu = {
	id = SPONGEMENU_CREDITS,
	name = "Credits",
	options = OptionList_Credits,
	cursor_pos = 0,
	spacing = 7,
	row_offset = -96,
	callback = "SM_OpenDefaultMenu"
}

local SM_Credits2Menu = {
	id = SPONGEMENU_CREDITS,
	name = "Credits",
	options = OptionList_Credits2,
	cursor_pos = 0,
	spacing = 7,
	row_offset = -96,
	callback = "SM_OpenDefaultMenu"
}

// Menu layout

local SM_MenuLayouts = {
	[SPONGEMENU_DEFAULT] = SM_DefaultMenu,
	[SPONGEMENU_GAMEPLAY] = SM_GameplayMenu,
	[SPONGEMENU_SECONDCOLOR] = SM_SecondcolorMenu,
	[SPONGEMENU_SOUNDS] = SM_SoundsMenu,
	[SPONGEMENU_CREDITS] = SM_CreditsMenu,
	[SPONGEMENU_CREDITS2] = SM_Credits2Menu
}

rawset(_G, "SM_ToggleCvar", function(player, param)

	local cvar = CV_FindVar(param)
	
	if (cvar == nil) then CONS_Printf("The specified cvar " + param + " does not exist!") return end

	if (cvar.value == 1) then
		COM_BufInsertText(player, cvar.name + " 0")
	else
		COM_BufInsertText(player, cvar.name + " 1")
	end
	
	player.ssvars.menu.delay = DELAY_TIME_BASIC
end)

local function SM_SetCvar(player, cvar, value)
	
	if (cvar == nil) then CONS_Printf(player, "The specified cvar does not exist!") return end
	
	player.ssvars.menu.delay = DELAY_TIME_BASIC

	COM_BufInsertText(player, cvar.name + " "  + value)
end

local function SM_CloseMenu(player)
	if not (player.ssvars.meleetimer) then
		player.ssvars.meleetimer = 8 // delay to prevent immediate chopping after exiting
	end
	player.ssvars.menu.active = false // and we are no longer active!
end

rawset(_G, "SM_OpenMenu", function(player, menu, pos)

	local current_menu = player.ssvars.menu

	if (current_menu.layout == menu) then CONS_Printf(player, "Hon hon! You are already in zat menu!") return end
	
	player.ssvars.menu.cursor_pos = pos or 0
	player.ssvars.menu.layout = SM_MenuLayouts[menu] // Set all variables to the next menu.
	player.ssvars.menu.delay = DELAY_TIME_BASIC
end)

rawset(_G, "SM_OpenDefaultMenu", function(player)
	SM_OpenMenu(player, SPONGEMENU_DEFAULT, player.ssvars.menu.layout.id - 1)
end)

rawset(_G, "SM_ExpandCredits", function(player)
	SM_OpenMenu(player, SPONGEMENU_CREDITS2, player.ssvars.menu.layout.id - 1)
end)

rawset(_G, "SM_InitalizeMenu", function(player)

	// Set back to default menu, if not already.
	if (player.ssvars.menu.layout != SM_DefaultMenu) then
		player.ssvars.menu.layout = SM_DefaultMenu
	end
	
	player.ssvars.menu.cursor_pos = 0 // Reset cursor position.
	
	player.ssvars.menu.active = true // Activate the menu.
end)

local function SM_GetCursorPos(player)
	return player.ssvars.menu.cursor_pos
end

local function SM_SetCursorPos(player, value, add)

	if (add) then
		player.ssvars.menu.cursor_pos = $ + value
	else
		player.ssvars.menu.cursor_pos = value
	end		
	
	S_StartSound(nil, sfx_ssmcur, player)
	
	player.ssvars.menu.delay = DELAY_TIME_BASIC // Set delay before moving cursor any further.
end

local function SM_GetLastValidColor()

	local lastcolor = SKINCOLOR_NONE

	for i = #skincolors - 1, SKINCOLOR_WHITE, -1 do
	
		if not (skincolors[i] and skincolors[i].accessible) then
			continue
		end
		
		lastcolor = i
		
		break
	end
	
	return lastcolor // If there are no other options, just default to None.
end

local function SM_GetNextValidColor(color)

	local nextcolor = SKINCOLOR_NONE
	local lastvalidcolor = SM_GetLastValidColor()
	
	// Default to SKINCOLOR_WHITE
	if (color == SKINCOLOR_NONE) then
		return SKINCOLOR_WHITE
	end

	for i = SKINCOLOR_WHITE, #skincolors - 1, 1 do
	
		if (i < color or skincolors[i].name == skincolors[color].name) then
			continue
		end
	
		if not (skincolors[i] and skincolors[i].accessible) then
			continue
		end
		
	    nextcolor = i
		
		break
	end
	
	return nextcolor // If there are no other options, just default to None.
end

local function SM_GetPreviousValidColor(color)

	local nextcolor = SKINCOLOR_NONE

	// Default to last known color in the skincolors table.
	if (color == SKINCOLOR_NONE) then
		return SM_GetLastValidColor()
	end

	for i = #skincolors - 1, SKINCOLOR_WHITE, -1 do
	
		if (i > color or i == color) then
			continue
		end
	
		if not (skincolors[i] and skincolors[i].accessible) then
			continue
		end
		
	    nextcolor = i
		
		break
	end
	
	return nextcolor // If there are no other options, just default to None.
end


local function SM_SetSkinColor(player, direction)

	local cvar
	local index
	local color_before
	local color_after
	local color_name
	local command
	
	index = player.ssvars.pants.color
	color_before = SM_GetPreviousValidColor(index)
	color_after = SM_GetNextValidColor(index)
	
	if (direction == 1) then // Right
		index = color_after
	elseif (direction == -1) then // Left
		index = color_before
	end
	
	// Set command depending on whether the player is the first or second player in game.
	if (splitscreen and player == secondarydisplayplayer) then
		command = "ss_pantscolor2"
	else
		command = "ss_pantscolor"
	end

	S_StartSound(nil, sfx_ssmsel, player)
	
	COM_BufInsertText(player, command + " " + index)
	
	player.ssvars.menu.delay = DELAY_TIME_FAST
end

local function SM_HandleSelection(player, inputs)

	local index = 0
	local layout = player.ssvars.menu.layout
	local options = layout and layout.options or nil
	local cursor_pos = player.ssvars.menu.cursor_pos

	// If the selection timer is running, stop processing input.
	if (player.ssvars.menu.delay > 0) then player.ssvars.menu.delay = $ - 1 return end

	if (layout == nil) then return end // can't navigate the menu without a layout
	
	local func = nil
	local value = nil
	local option = options and options[cursor_pos+ 1] or nil
	local max_options = options and #options - 1 or 0 

	// The player could be exiting out of the menu, so check the SPIN button first.
	if (inputs.spin)
		if (layout.callback != nil) then
			func = layout.callback
		else
			SM_CloseMenu(player) // exit out of menu entirely
		end
		S_StartSound(nil, sfx_ssmok, player)
	elseif (inputs.jump and option and option.type != 2 and option.func != nil) // the entry has a valid function
		func = option.func
		value = option.param
	end
	
	if (_G[func] != nil) then S_StartSound(nil, sfx_ssmsel, player) _G[func](player, value) return end // if it exists, execute the function

	// vertical selection
	if (inputs.forwards and SM_GetCursorPos(player) > 0) then
		SM_SetCursorPos(player, -1, true) // going back up
		if (SM_GetCursorPos(player) < 0) then
			SM_SetCursorPos(player, 0) // if cursor is too far out of position, set it back.
		end
	elseif (inputs.backwards and SM_GetCursorPos(player) < max_options)
		SM_SetCursorPos(player, 1, true) // going down
		if (SM_GetCursorPos(player) > max_options) 
			SM_SetCursorPos(player, max_options) // if cursor is too far out of position, set it back.
		end 
	end
		
	if not (option and option.type == OPT_TYPE_MULTI) then return end // horizontal selection only
		
	local cvar = (option.param != nil and CV_FindVar(option.param)) or nil

	// for skincolors, handle the changing of colors in a separate function.
	if (option.variable == "skincolor")
		if (inputs.right) then
			SM_SetSkinColor(player, 1)
		elseif (inputs.left) then
			SM_SetSkinColor(player, -1)
		end
	// for cvars, look through the table of items for available values
	elseif (option.variable == "cvar" and option.items != nil and cvar != nil) then	
		if (inputs.right 
		and option.items[(cvar.value + 1) + 1] != nil 
		and cvar.value < #(option.items)) then
			S_StartSound(nil, sfx_ssmsel, player)
			SM_SetCvar(player, cvar, option.items[(cvar.value + 1) + 1]) // right
		elseif (inputs.left 
		and option.items[(cvar.value + 1) - 1] != nil 
		and cvar.value > 0) then
			S_StartSound(nil, sfx_ssmsel, player)
			SM_SetCvar(player, cvar, option.items[(cvar.value + 1) - 1]) // left
		end
	end
end

local function StopPlayerInput()
	for p in players.iterate() do
		if not (p.ssvars) then continue end
		if not (p.ssvars and p.ssvars.menu.active) then continue end
		if (p.playerstate == PST_DEAD) then SM_CloseMenu(p) continue end
		
		local inputs = {
			forwards = p.cmd.forwardmove > 0,
			backwards = p.cmd.forwardmove < 0,
			left = p.cmd.sidemove < 0,
			right = p.cmd.sidemove > 0,
			jump = (p.cmd.buttons & BT_JUMP),
			spin = (p.cmd.buttons & BT_SPIN)
		}
		
		p.cmd.forwardmove = 0
		p.cmd.sidemove = 0
		p.cmd.buttons = 0
		
		p.aiming = 0 // set aiming to prevent from looking up or down while menu is active
		p.mo.angle = p.drawangle // set angle to drawangle to prevent looking left or right
		
		SM_HandleSelection(p, inputs)
	end
end
addHook("PreThinkFrame", StopPlayerInput)

local function GetEmptyText(string, diff)

	local text = " "

	if not (string) then return " " end

	for i=1, string:len() - diff do
		text = $ + " " 
	end
	
	return text
end

local function SM_DrawMenuText(v, player, x, y)

	local layout
	local cursor_pos
	local entries
	local flags
	local x_offset
	local y_offset
	
	if (player.ssvars.menu.layout == nil) then 
		return // non-existent
	end
	
	layout = player.ssvars.menu.layout
	cursor_pos = player.ssvars.menu.cursor_pos
	flags = V_PERPLAYER // Flags
	entries = layout.options or nil
	x_offset = layout.row_offset or 0
	y_offset = layout.spacing or 0

	// Header Text 
	customhud.CustomFontString(v, x, y, layout.name or "-", "SBPFN", flags, "center", nil, 0) 
	
	// Entry Text
	if (entries) then
		for i = 1, #entries do
		
			local name = entries[i].name or "-"
			local variable = nil
			local scale = entries[i].scale != nil and entries[i].scale or 7*FRACUNIT/10
			local param = entries[i].param != nil and entries[i].param or nil
			local length = name:len()
				
			 // If the variable is a CVAR, get it's string value.
			if (entries[i].variable == "cvar" and param and CV_FindVar(param) != nil) then
				variable = (CV_FindVar(param)).string
			end
			
			// Secondcolor display handling
			if (entries[i].variable == "skincolor") then
			
				local sprite
				local pants
				local color
			
				if (player.ssvars.pants.color == SKINCOLOR_NONE) then	
					variable = "None"
				else
					variable = R_GetNameByColor(player.ssvars.pants.color)
				end
			
				sprite = v.getSprite2Patch("supersponge", "STND", false, A, 0)
				pants = v.getSpritePatch("01P@", A, 0)
				color = (variable != "None" and R_GetColorByName(variable)) or 0
		
				// draw the player sprite 
				v.drawScaled(160*FRACUNIT, 138*FRACUNIT, FRACUNIT/4, sprite, flags, 
				v.getColormap("supersponge", player.mo.color))
				
				// draw the pants sprite
				if (color) then
					v.drawScaled(160*FRACUNIT, 138*FRACUNIT, FRACUNIT/4, pants, flags, 
					v.getColormap("supersponge", color))
				end
			end
			
			// display the corresponding variable for "X" depending on the entry type and variable.
			if (variable) then
				// draw the variable's value - usually "On" or "Off"
				if (entries[i].type == OPT_TYPE_TOGGLE)  then
					customhud.CustomFontString(v, x*FRACUNIT + 64*FRACUNIT, (y + 8 + y_offset * i)*FRACUNIT, variable, 
					"SBPFN", flags, "left", scale, 0)						
				elseif (entries[i].type == OPT_TYPE_MULTI) then // draw a string with two arrows.
					
					local text_x_offset = x*FRACUNIT + 64*FRACUNIT 
					local text_y_offset = variable:len() > 10 and (y + 10 + y_offset * i)*FRACUNIT or (y + 8 + y_offset * i)*FRACUNIT
					local text_scale = variable:len() > 10 and scale * 7/10 or scale
					local arrow_offset = variable:len() > 10 and (8 * variable:len()/8)*FRACUNIT or (8 * variable:len()/3)*FRACUNIT
					
					customhud.CustomFontString(v, text_x_offset - arrow_offset - 8*FRACUNIT, (y + 8 + y_offset * i)*FRACUNIT, 
					"<" + GetEmptyText(variable, 8), "SBPFN", flags, "center", scale, 0) // Left
					
					customhud.CustomFontString(v, text_x_offset, text_y_offset, 
					variable, "SBPFN", flags, "center", text_scale, 0) // Middle
					
					customhud.CustomFontString(v, text_x_offset + arrow_offset + 8*FRACUNIT, (y + 8 + y_offset * i)*FRACUNIT, 
					GetEmptyText(variable, 8) + ">", "SBPFN", flags, "center", scale, 0) // Right
				end
			end
			
			// If we're just drawing text, do not draw the name.
			if (entries[i].type == OPT_TYPE_TEXT and type(param) == "table" and param != {}) then
				for t = 1, #param, 1 do
					customhud.CustomFontString(v, (x + x_offset)*FRACUNIT, (y + y_offset * i + (12 * t - 1))*FRACUNIT,
					param[t], "SBPFN", flags, "left", scale, 0)
				end
			end
		
			if (entries[i].type == OPT_TYPE_TEXT) then continue end // already done drawing text
			
			local cursor_x = x + x_offset - FixedMul(length * 4, scale) - 4 // cursor x offset
			local cursor_y = y + 8 + y_offset * i // cursor y offset
			
			// If there are multiple entries, draw an arrow to the left of the most currently selected one.
			if (#entries > 1 and cursor_pos == i - 1) then
				customhud.CustomFontString(v, cursor_x*FRACUNIT, cursor_y*FRACUNIT, ">", "SBPFN", flags, "center", scale, 0)
			end
			
			customhud.CustomFontString(v, (x + x_offset)*FRACUNIT, (y + 8 + y_offset * i)*FRACUNIT, "  " + name, "SBPFN", flags, "center", scale, 0)				
		end
	end
	
	// Back and Start Text
	customhud.CustomFontString(v, 98*FRACUNIT, 144*FRACUNIT, "Jump - Next", "SBPFN", flags, "center", 7*FRACUNIT/10, 0) 
	customhud.CustomFontString(v, 236*FRACUNIT, 144*FRACUNIT, "Spin - Back", "SBPFN", flags, "center", 7*FRACUNIT/10, 0) 
end

local function SB_DrawSpongeMenu(v, player, cam)

	local flags

	if not (player.mo and player.mo.valid and player.ssvars.menu.active) then 
		return // not up
	end
	
	// Border Background
	v.drawScaled(33*FRACUNIT/8, 19*FRACUNIT/8, FRACUNIT, v.cachePatch(OPT_MENU_BACK), V_PERPLAYER|V_TRANSLUCENT, nil)

	// Border
	v.drawScaled(33*FRACUNIT/8, 19*FRACUNIT/8, FRACUNIT, v.cachePatch(OPT_MENU_BORDER), V_PERPLAYER, nil)

	// Text
	SM_DrawMenuText(v, player, 164, 46)
end
customhud.SetupItem("supersponge_menu", "supersponge", SB_DrawSpongeMenu, "game", 1)


