// JELLYFISHING NET


// P_GetNetCount
//
// Returns the count of items from a fishing net.
//
rawset(_G, "P_GetNetCount", function(sponge)
	local count = 0
	local things = sponge.ssvars.ammo["things"] and sponge.ssvars.ammo["things"][SS_POWERUP_NET] or {}
	for i = 1, supersponge_powerup_info[SS_POWERUP_NET].maxammo, 1 do
		if (things[i] != nil) then
			count = $ + 1
		end
	end
	return count
end)


// P_AddNetItem
//
// Adds an item to the jellyfishing net
//
local function P_AddNetItem(sponge, item)
	
	local mobj = {}
	local things = sponge.ssvars.ammo and sponge.ssvars.ammo["things"] and sponge.ssvars.ammo["things"][SS_POWERUP_NET] or {}
	local oldtype = item.extravalue3
	local maxammo = supersponge_powerup_info[SS_POWERUP_NET].maxammo
	
	// Already full?!
	if (things != {} and #things >= maxammo) then CONS_Printf(sponge, "Your net is full!") return end	
	
	// Reset item type back to default if not already.
	if (item.type == MT_SUPERSPONGE_FAKEMISSILE and oldtype) then
		item.type = oldtype
	end
	
	mobj = {
		type = item.type,
		sprite = item.sprite,
		flags = item.flags,
		color = item.color,
		colorized = item.colorized,
		scale = item.scale,
		deathstate = mobjinfo[item.type].deathstate
	}
	
	table.insert(sponge.ssvars.ammo["things"][SS_POWERUP_NET], mobj)	
end

// P_RemoveNetItem
//
// Removes an item from the fishing net
//
local function P_RemoveNetItem(sponge)

	local things = sponge.ssvars.ammo and sponge.ssvars.ammo["things"] and sponge.ssvars.ammo["things"][SS_POWERUP_NET] or {}
	
	// Nothing inside.
	if (things == {}) then CONS_Printf(sponge, "Your jellyfishing net is empty!") return end
	
	for i = supersponge_powerup_info[SS_POWERUP_NET].maxammo, #things, -1 do
		
		if (things[i] == nil) then 
			continue 
		end
		
		things[i] = nil // Found a valid item slot.
		
		break
	end
end

// P_GetLastNetItem
//
// Returns the last item from the fishing net.
//
local function P_GetLastNetItem(sponge)

	local things = sponge.ssvars.ammo and sponge.ssvars.ammo["things"] and sponge.ssvars.ammo["things"][SS_POWERUP_NET] or {}
	
	// Nothing inside.
	if (things == {}) then CONS_Printf(sponge, "Your jellyfishing net is empty!") return nil end
	
	for i = supersponge_powerup_info[SS_POWERUP_NET].maxammo, #things, -1 do
		// Found an item.
		if (things[i] != nil) then
			return things[i]
		end
	end
	
	CONS_Printf("Could not find any valid items...")
	
	return nil
end

rawset(_G, "P_LaunchNetItem", function(sponge)

	local item
	local item_type
	local rocket
		
	// Fire any stored objects if the jellyfishing net is not empty
	if (P_GetNetCount(sponge) > 0) then
	
		sponge.weapondelay = 1
		sponge.drawangle = sponge.mo.angle
		
		item = P_GetLastNetItem(sponge)
		
		P_SpawnSpongeMissile(sponge, item, true)
		
		P_RemoveNetItem(sponge)
		
		sponge.ssvars.shoottimer = SS_SHOOT_COOLDOWN/4
	end
end)

// P_UseJellyfishNet
//
// Handles the usage of Spongebob's jellyfishing net.
//
rawset(_G, "P_UseJellyfishNet", function(sponge, shooting)
	
	// Set melee timer
	sponge.ssvars.meleetimer = SS_KARATE_COOLDOWN
	
	// Set state
	sponge.mo.state = S_PLAY_SSNET
	
	if (shooting) then
		sponge.ssvars.net.mode = 1 
	else
		sponge.ssvars.net.mode = 0
	end
	
	S_StartSound(sponge.mo, sfx_sbnet)
	
	sponge.ssvars.fishing = true
end)

local function IsMobjCapturable(sponge, target) 

	local type
	local state
	local flags
	local height
	local capture
	
	capture = true
	
	type = target.type
	state = target.state
	flags = target.flags
	height = target.height
	
	// The net is full!
	if (P_GetNetCount(sponge) >= supersponge_powerup_info[SS_POWERUP_NET].maxammo) then
		return false
	end
	
	// The target no longer exists.
	if not (target and target.valid) then
		return false
	end
	
	// The target is dead.
	if (target.health <= 0) then
		return false
	end
	
	// Don't capture these things.
	if (type == MT_PLAYER)
	or (type == MT_SHOCKWAVE)
	or (type == MT_SUPERSPONGE_POWERUP)
	or (type == MT_SUPERSPONGE_VACBOX)
	or (type == MT_SUPERSPONGE_NETBOX)
	or (type == MT_SUPERSPONGE_BUBBLE) then
		capture = false
	end
	
	// Never hit a sparring dummy.
	if (CBW_Battle != nil and type == MT_SPARRINGDUMMY) then
		capture = false
	end
	
	// Don't capture things without these flags
	if not P_IsMobjCollectible(target) and not (target.flags & (MF_ENEMY|MF_MISSILE|MF_MONITOR)) then
		capture = false
	end
	
	// The target could be any kind of shockwave.
	if (states[state].action == A_Boss3ShockThink) then
		capture = false
	end
	
	// Dont capture things that are bigger than 48 fracunits
	if (height > 48*FRACUNIT) then
		capture = false
	end

	return capture
end

// P_SSNetCapture
//
// Special items handler for the fishing net
//
rawset(_G, "P_SSNetCapture", function(net, sponge, target)

	local type
	
	if not IsMobjCapturable(sponge, target) then
		return false
	end
	
	type = target.type
	
	// "Burn" the net upon colliding with a burning Pyrefly
	if (target.type == MT_PYREFLY and target.state == S_PYREFLY_BURN) then
	
		local flame
		local x, y, z 
		
		// Set to previous coordinates of netbox before torching
		x = net.x
		y = net.y
		z = net.z
		
		P_RemoveSSPowerup(sponge, SS_POWERUP_NET, false)
		
		CONS_Printf(sponge, "Your net was burned by Pyrefly's flames.")
		
		if (net and net.valid) then
			P_RemoveMobj(net)
		end
		
		// End immediately.
		if (sponge.mo.state != S_PLAY_FALL) then
			sponge.mo.state = S_PLAY_FALL 
		end
	
		flame = P_SpawnMobj(x, y, z, MT_FLAME)
		flame.flags = 0
		flame.fuse = 35
		
		S_StartSound(flame, sfx_fire)

		return true
	end
	
	// "Break" the net upon colliding with a Green Snapper.
	if (target.type == MT_GSNAPPER) then
		
		local x, y, z
		local dummy
		
		x = net.x
		y = net.y
		z = net.z
		
		P_RemoveSSPowerup(sponge, SS_POWERUP_NET, false)
		
		CONS_Printf(sponge, "Your net was torn apart by Green Snapper's spikes.")
		
		if (net and net.valid) then
			P_RemoveMobj(net)
		end
		
		// End immediately.
		if (sponge.mo.state != S_PLAY_FALL) then
			sponge.mo.state = S_PLAY_FALL 
		end
		
		// Spawn SFX dummy
		dummy = P_SpawnMobj(x, y, z, MT_RAY)
		dummy.state = S_THOK
		dummy.tics = 35
		
		S_StartSound(dummy, sfx_wbreak)

		return true
	end
	
	// Spawn collectibles
	if P_IsMobjCollectible(target) then P_SpawnMobj(sponge.mo.x, sponge.mo.y, sponge.mo.z, type) end	
	
	// Kill monitors
	if (target.flags & MF_MONITOR) then P_KillMobj(target, net, sponge.mo, 0) return true end
	
	// Check if the net is full before adding and make sure it is a enemy or missile.
	if (P_GetNetCount(sponge) < supersponge_powerup_info[SS_POWERUP_NET].maxammo and target.flags & (MF_ENEMY|MF_MISSILE))
		P_AddNetItem(sponge, target)
	end
	
	if (target.type == MT_PYREFLY) then
		P_RemovePyreFly(target) // Specialized removal for Pyreflies.
	elseif (target.type == MT_CRUSHSTACEAN or target.type == MT_BANPYURA) then
		P_RemoveSegmentedEnemy(target) // Specialized removal for multi-segmented enemies.
	else
		P_RemoveMobj(target)
	end
	
	return true
end)