local total_space_suits = 0
local pickups_enabled = CV_FindVar("ss_pickups")
local powerups_enabled = CV_FindVar("ss_powerups")

// The main monitor table which decides which box corresponds to which powerup.
rawset(_G, "supersponge_monitor_table", {
	["MT_PITY_BOX"] = SS_POWERUP_GLASSES,
	["MT_PITY_GOLDBOX"] = SS_POWERUP_GLASSES,
	["MT_ATTRACT_BOX"] = SS_POWERUP_NET,
	["MT_ATTRACT_GOLDBOX"] = SS_POWERUP_NET,
	["MT_FORCE_BOX"] = SS_POWERUP_BLOWER,
	["MT_FORCE_GOLDBOX"] = SS_POWERUP_BLOWER,	
	["MT_ARMAGEDDON_BOX"] = SS_POWERUP_PIE,
	["MT_ARMAGEDDON_GOLDBOX"] = SS_POWERUP_PIE,
	["MT_WHIRLWIND_BOX"] = SS_POWERUP_BALLOON,
	["MT_WHIRLWIND_GOLDBOX"] = SS_POWERUP_BALLOON,
	["MT_ELEMENTAL_BOX"] = SS_POWERUP_SUIT,
	["MT_ELEMENTAL_GOLDBOX"] = SS_POWERUP_SUIT,
	["MT_FLAMEAURA_BOX"] = SS_POWERUP_LAUNCHER,
	["MT_FLAMEAURA_GOLDBOX"] = SS_POWERUP_LAUNCHER,
	["MT_BUBBLEWRAP_BOX"] = SS_POWERUP_WAND,
	["MT_BUBBLEWRAP_GOLDBOX"] = SS_POWERUP_WAND,
	["MT_THUNDERCOIN_BOX"] = SS_POWERUP_BOOTS,
	["MT_THUNDERCOIN_GOLDBOX"] = SS_POWERUP_BOOTS,
	["MT_1UP_BOX"] = SS_POWERUP_UNDERPANTS
})

// An extra table for powerup icons...? Who cares!
rawset(_G, "supersponge_boxicon_table", {
	[MT_WHIRLWIND_ICON] = SS_POWERUP_BALLOON,
	[MT_ELEMENTAL_ICON] = SS_POWERUP_SUIT,
	[MT_ATTRACT_ICON] = SS_POWERUP_NET,
	[MT_FORCE_ICON] = SS_POWERUP_BLOWER,
//	[MT_1UP_ICON] = SS_POWERUP_UNDERPANTS, // For life monitor, just use default behavior.
	[MT_FLAMEAURA_ICON] = SS_POWERUP_LAUNCHER,
	[MT_BUBBLEWRAP_ICON] = SS_POWERUP_WAND,
	[MT_ARMAGEDDON_ICON] = SS_POWERUP_PIE
})

rawset(_G, "supersponge_boxsprites_table", {
	[SS_POWERUP_BALLOON] = {sprite = SPR_SSD5, frame = A},
	[SS_POWERUP_BLOWER] = {sprite = SPR_SSD5, frame = B},
	[SS_POWERUP_NET] = {sprite = SPR_SSD5, frame = C},
	[SS_POWERUP_LAUNCHER] = {sprite = SPR_SSD5, frame = D},
	[SS_POWERUP_WAND] = {sprite = SPR_SSD5, frame = E},
	[SS_POWERUP_BOOTS] = {sprite = SPR_SSD5, frame = F},
	[SS_POWERUP_HELMET] = {sprite = SPR_SSD5, frame = G},
	[SS_POWERUP_GLASSES] = {sprite = SPR_SSD5, frame = H},
	[SS_POWERUP_SUIT] = {sprite = SPR_SSD5, frame = I},
	[SS_POWERUP_PIE] = {sprite = SPR_SSD5, frame = J}
})

rawset(_G, "IsRingMonitor", function(monitor)
	return (monitor.type == MT_RING_BOX or mobjinfo[monitor.type].damage == MT_RING_ICON)
end)

rawset(_G, "BoxOnlyHasSSIcon", function(monitor)
	return (IsRingMonitor(monitor) or monitor.type == MT_SNEAKERS_BOX or monitor.type == MT_SNEAKERS_GOLDBOX)
end)


rawset(_G, "IsSuperSpongeMonitor", function(monitor)
	if monitor == nil then return false end
	for mtype,v in pairs(supersponge_monitor_table) do
		if (_G[mtype] == monitor.type) then return true end // try to see if the monitor's type exists in the table
	end
	return false
end)

rawset(_G, "IsGoldenMonitor", function(monitor)

	local golden

	for mtype,v in pairs(supersponge_monitor_table) do
	    golden = (_G[mtype] == monitor.type and mtype:match("GOLDBOX"))
		if (golden) then
			break
		end
	end
	
	return golden
end)

rawset(_G, "DisplaySSPickupIcon", function(monitor)
	if not (monitor and monitor.valid) then return end
	if not (monitor.flags & MF_MONITOR and monitor.type != MT_1UP_BOX) then return end // don't target 1UP boxes
	if not (IsSuperSpongeMonitor(monitor) or BoxOnlyHasSSIcon(monitor)) then return end // only look for compatible monitors
	if (monitor.tracer and monitor.tracer.valid) then return end
	monitor.tracer = P_SpawnMobj(monitor.x, monitor.y, monitor.z, MT_OVERLAY)
	monitor.tracer.target = monitor
	monitor.tracer.state = S_SUPERSPONGE_ICON1
	if BoxOnlyHasSSIcon(monitor) then return else monitor.powerup = GetSSPickupFromMonitor(monitor) end
end)

addHook("MobjSpawn", function(monitor)
	if not (monitor and monitor.valid and (monitor.flags & MF_MONITOR)) then
		return
	end

	if IsGoldenMonitor(monitor) then
		monitor.is_golden = true
	end

	DisplaySSPickupIcon(monitor)
end)

local function RingsToSpatulas(spatula)
	// incomprehensible gibberish
	if not (not multiplayer 
	and (consoleplayer and consoleplayer.valid) 
	and P_IsSuperSponge(consoleplayer.skin)) then
		// revert if multiplayer or not spongebob
		if (mobjinfo[spatula.type].deathsound == sfx_sbgold) then
			mobjinfo[spatula.type].deathsound = sfx_itemup
		end
		return 
	end
	spatula.state = S_SUPERSPONGE_SPATULA
	if (mobjinfo[spatula.type].deathsound != sfx_sbgold)
		mobjinfo[spatula.type].deathsound = sfx_sbgold
	end
end
addHook("MobjSpawn", RingsToSpatulas, MT_RING)
addHook("MobjSpawn", RingsToSpatulas, MT_FLINGRING)

local function BoxIconToSSBoxIcon(icon)
	// incomprehensible gibberish
	if not (not multiplayer 
	and (consoleplayer and consoleplayer.valid) 
	and P_IsSuperSponge(consoleplayer.skin)) then
		return 
	end
	if (icon.type == MT_RING_ICON) then
		icon.state = S_SUPERSPONGE_SPATULA_ICON1
	elseif (icon.type == MT_SNEAKERS_ICON)
		icon.state = S_SUPERSPONGE_QUICKSTER_ICON1
	end
end
addHook("MobjSpawn", BoxIconToSSBoxIcon, MT_RING_ICON)
addHook("MobjSpawn", BoxIconToSSBoxIcon, MT_SNEAKERS_ICON)

local function TokensToTokens(token)
	// incomprehensible gibberish
	if not (not multiplayer 
	and (consoleplayer and consoleplayer.valid) 
	and P_IsSuperSponge(consoleplayer.skin)) then
		return 
	end
	token.state = S_SUPERSPONGE_TOKEN // not setting any sounds because of hardcoded token sfx
end
addHook("MobjSpawn", TokensToTokens, MT_TOKEN)

local function SpatulaDeath(spatula)
	// incomprehensible gibberish
	if not (not multiplayer 
	and (consoleplayer and consoleplayer.valid) 
	and P_IsSuperSponge(consoleplayer.skin)) then
		return 
	end
	
	// level could be loaded with rings
	if not (spatula.sprite == SPR_SSD1) then 
		return
	end
	
	spatula.state = S_SUPERSPONGE_SPRK // spatulas disappear immediately like in SuperSponge

	return true
end
addHook("MobjDeath", SpatulaDeath, MT_RING)
addHook("MobjDeath", SpatulaDeath, MT_FLINGRING)

local function IsSpaceOrLavaLevel()
	
	local need_it = false
	local space_sky = false
	
	// First, let's do the simplest thing and check for the map's skynum
	if (levelskynum == 21) // Old ERZ sky
	or (levelskynum == 22) // ERZ sky
	or (levelskynum >= 50 and levelskynum <= 59) // Special Stage sky
	or (levelskynum == 98) // BHZ (Alt.) Sky
	or (levelskynum == 99) // BHZ / Tutorial sky
		space_sky = true
	end
	
	// Next, let's check EVERY sector for something that can do lava damage or has a space countdown sector.
	for sec in sectors.iterate do
	
		local in_space = (udmf and (sec.specialflags & SSF_OUTERSPACE) or GetSecSpecial(sec.special, 1) == 12) and space_sky
		local has_lava = (udmf and (sec.damagetype == SD_LAVA) or GetSecSpecial(sec.special, 1) == 3)
		
		if not (in_space or has_lava) then 
			continue 
		end
				
		need_it = true 
		
		break
	end
	
	return need_it
end

rawset(_G, "P_GetSSPickupFromShield", function(shield)
	for i = 1, #supersponge_powerup_info, 1 do
		if not supersponge_powerup_info[i] then continue end
		if not (supersponge_powerup_info[i].shield) then continue end
		if (supersponge_powerup_info[i].shield == shield) then
			return i
		end
	end
	return -1 // No shield...
end)

rawset(_G, "GetSSPickupFromMonitor", function(monitor)

	local pickup = -1
	local id = monitor.unique_id != nil and monitor.unique_id or 1
	local in_space_or_lava = IsSpaceOrLavaLevel()
	
	for mtype,v in pairs(supersponge_monitor_table) do
	    if (_G[mtype] == monitor.type) then pickup = v break end
	end
	
	// Warn the player, if no pickup was found.
	if (pickup == -1) then
		CONS_Printf(consoleplayer, "No valid pickup was found for monitor of type: " + monitor.type)
	end
	
	// Only replace the suit in levels where it is not present or it is the second consecutive powerup.
	if (pickup == SS_POWERUP_SUIT and not (in_space_or_lava or id and id % 2 == 0))
		pickup = SS_POWERUP_LAUNCHER
	end
	
	return pickup or SS_POWERUP_BALLOON
end)

rawset(_G, "P_SpawnSSPickup", function(x, y, z, powerup, type)

	local pickup

	pickup = P_SpawnMobj(x, y, z, MT_SUPERSPONGE_PICKUP)
	
	// Set powerup.
	if (powerup) then
		pickup.powerup = powerup 
	else
		pickup.powerup = SS_POWERUP_BALLOON
	end
	
	// Set certain flags depending on the pickup's type.
	if (type == SS_PKTYPE_RESPAWN) then 
		pickup.is_golden = true
	elseif (type == SS_PKTYPE_DESPAWN) then
		pickup.fuse = 30*TICRATE
	end
	
	return pickup
end)

rawset(_G, "ToggleSSPickups", function(player, on)
	
	local pickups_enabled = CV_FindVar("ss_pickups")
	local powerups_enabled = CV_FindVar("ss_powerups")
	
	if (multiplayer) then return end // only toggle visibility of placed pickups outside of multiplayer
	
	total_space_suits = 0
	
	for pickup in mobjs.iterate() do
		
		local show_pickup = true
		local is_pickup = false
		local is_dropped = pickup.fuse > 0

		// Ignore dead monitors / pickups
		if not (pickup and pickup.valid and pickup.health > 0) then
			continue
		end
		
		// Hide monitors and pickups only.
		if not (pickup.flags & MF_MONITOR) and not (pickup.type == MT_SUPERSPONGE_PICKUP) then
			continue 
		end
		
		// Look for compatible monitors in the map.
		if ((pickup.flags & MF_MONITOR) and not IsSuperSpongeMonitor(pickup)) then 
			continue // If the mobj is a monitor, it must be one that corresponds to a pickup.
		end
	
		// convert monitors to pickups
		if (on and pickup.type != MT_SUPERSPONGE_PICKUP) then
		
			local oldtype = pickup.type
			local powerup = GetSSPickupFromMonitor(pickup)
			
			// Remove 1UP overlay, if possible.
			if (pickup.tracer and pickup.tracer.valid) then
				P_RemoveMobj(pickup.tracer)
			end
			
			// Set pickup ID for space suit
			if (supersponge_monitor_table[pickup.type] == SS_POWERUP_SUIT) then
				pickup.unique_id = total_space_suits + 1
			end
			
			pickup.type = MT_SUPERSPONGE_PICKUP
			pickup.flags = mobjinfo[pickup.type].flags
			pickup.state = mobjinfo[pickup.type].spawnstate
			pickup.powerup = powerup
			pickup.monitor = oldtype
		
		elseif (not on and pickup.type == MT_SUPERSPONGE_PICKUP and pickup.monitor != nil) then
			pickup.type = pickup.monitor
			pickup.flags = mobjinfo[pickup.type].flags
			pickup.state = mobjinfo[pickup.type].spawnstate
		end
		
		if (not on and pickup.flags & MF_MONITOR and pickup.type != MT_1UP_BOX) then DisplaySSPickupIcon(pickup) end // re-display powerup icon
	end
end)