// POWERUP STUFF

local cv_powerups = CV_FindVar("ss_powerups")

rawset(_G, "P_GetSSPowerupCount", function(player, filter)
	
	local limit
	local count
	
	count = 0
	
	for id = 1, #supersponge_powerup_info, 1 do
	
		// The entry does not exist.
		if (supersponge_powerup_info[id] == nil) then continue end
	
		// Ignore other types unless given in filter
		if (filter != nil and filter != supersponge_powerup_info[id].type) then continue end
	
		// Add to count of powerups.
		if (player.ssvars.powerups[id]) count = $ + 1 end 
	end
	
	return count
end)

rawset(_G, "P_SpawnSSPowerup", function(x, y, z, id)

	local mobj
	local nonexistent = (id <= 0 or supersponge_powerup_info[id] == nil)

	if (nonexistent) then print("Powerup with ID " + id + " does not exist!") return end
	
	mobj = P_SpawnMobj(x, y, z, MT_SUPERSPONGE_POWERUP)
	mobj.powerup = id
	mobj.fuse = 30*TICRATE // Every "dropped" powerup lasts at least 30 seconds.
	mobj.scale = supersponge_powerup_info[id].scale
	
	return mobj
end)

rawset(_G, "P_DoSSPowerupSwap", function(player, mobj, angle)

	local id
	local tbl
	local pthing 
	local x, y, z
	local ammo
	local ammotype
	local maxammo

	tbl = supersponge_powerup_info
	
	if (tbl[mobj.powerup] == nil) then CONS_Printf(player, "Powerup with ID " + id + " does not exist!") return false end
	
	if (tbl[mobj.powerup] and tbl[mobj.powerup].type == SS_TYPE_GEAR) then
		return false
	end
	
	ammo = tbl[mobj.powerup].ammo or false
	ammotype = tbl[mobj.powerup].ammotype or "none"
	maxammo = tbl[mobj.powerup].maxammo or 0
	
	id = mobj.powerup or SS_POWERUP_BALLOON
	
	if (angle == nil) then
		angle = player.drawangle
	end
	
	x = player.mo.x + (16*FRACUNIT>>FRACBITS) * cos(player.drawangle)
	y = player.mo.y + (16*FRACUNIT>>FRACBITS) * sin(player.drawangle)
	z = player.mo.z + player.mo.height/2
	
	pthing = P_SpawnSSPowerup(x, y, z, id)
	
	if not (pthing and pthing.valid) then
		return false
	end
	
	pthing.scale = FixedMul(tbl[id].scale, player.mo.scale)
	pthing.owner = player // set previous owner because why not
	
	// swap ammo from picked up weapon to player
	if (ammo and ammotype and ammotype:lower() != "none") then
		// check special ammo for dropped net / blower items
		if (ammotype == "things") then
			pthing.things = player.ssvars.ammo[ammotype] and player.ssvars.ammo[ammotype][id] or {}
			player.ssvars.ammo[ammotype][id] = mobj.things != nil and mobj.things or {}
		else
			pthing.ammo = player.ssvars.ammo[ammotype]
			if (mobj.ammo != nil) then
				player.ssvars.ammo[ammotype] = mobj.ammo
			else
				player.ssvars.ammo[ammotype] = maxammo or 0
			end
		end
	end
	
	// Thrust 8*FRACUNITS forwards
	P_Thrust(pthing, angle, 4*FRACUNIT)
	// Thurst 8*FRACUNITS vertically
	P_SetObjectMomZ(pthing, 2*FRACUNIT, true)
	
	return true
end)

local function P_DoPowerupToss(player, id, angle)
	
	local tbl
	local pthing 
	local x, y, z
	local ammo
	local ammotype
	
	if not (id) then
		return
	end
	
	tbl = supersponge_powerup_info
	
	if (angle == nil) then
		angle = player.drawangle
	end
	
	x = player.mo.x + (16*FRACUNIT>>FRACBITS) * cos(player.drawangle)
	y = player.mo.y + (16*FRACUNIT>>FRACBITS) * sin(player.drawangle)
	z = player.mo.z + player.mo.height/2
	
	ammo = tbl[id].ammo or false
	ammotype = tbl[id].ammotype or "none"
	
	pthing = P_SpawnSSPowerup(x, y, z, id)
	
	if not (pthing and pthing.valid) then
		return
	end
	
	if (P_MobjFlip(player.mo) == -1) then
		pthing.flags2 = $|MF2_OBJECTFLIP
	end
	
	pthing.scale = FixedMul(tbl[id].scale, player.mo.scale)
	pthing.owner = player // set previous owner because why not
	
	// carry over ammo to dropped weapon
	if (ammo and ammotype and ammotype:lower() != "none") then	
		// handle special ammo for net / blower items
		if (ammotype == "things") then
			pthing.things = player.ssvars.ammo["things"][id] or {}
			player.ssvars.ammo["things"][id] = {}
		else
			pthing.ammo = player.ssvars.ammo[ammotype]			
			player.ssvars.ammo[ammotype] = 0
		end
	end
	
	// Thrust 8*FRACUNITS forwards
	P_Thrust(pthing, angle, 4*FRACUNIT)
	
	// Thurst 8*FRACUNITS vertically
	P_SetObjectMomZ(pthing, (pthing.flags2 & MF2_OBJECTFLIP) and -2*FRACUNIT or 2*FRACUNIT, true)
end

// P_RemoveSSPowerup
// Removes a single specified powerup from the player's array
rawset(_G, "P_RemoveSSPowerup", function(player, id, toss, randangle)

	local tbl
	local limit
	local angle
	local ammo
	local ammotype
	
	tbl = supersponge_powerup_info
	
	if (tbl[id] == nil) then return end // does not exist

	if not (player.ssvars.powerups[id]) then return end // not active

	player.ssvars.powerups[id] = 0
		
	// toss the item with a set angle
	if (toss) then
	
		// randomized angle toss
		if (randangle)
			angle = FixedAngle(P_RandomRange(0, 315)*FRACUNIT)
		else
			angle = player.drawangle
		end	
		
		P_DoPowerupToss(player, id, angle)
	else
		ammo = tbl[id].ammo or nil
		ammotype = tbl[id].ammotype or nil
		if not (ammo and ammotype) then 
			return // no ammo
		end
		if not (ammotype and ammotype:lower() != "none") then
			return // no defined ammo type
		end
		// Reset any player ammo associated with the weapon.
		if (ammotype == "things")
			if (player.ssvars.ammo["things"] == nil) then
				player.ssvars.ammo["things"] = {}
			end
			player.ssvars.ammo["things"][id] = {}
		else
			player.ssvars.ammo[ammotype] = 0
		end
	end
end)

rawset(_G, "P_IsPowerupActive", function(player, powerup)
	
	local active = true
	
	// It is not active if SRB2infinity is being played, powerups are turned off or the player does not have it.
	if (PSOIsEnabled() or not (cv_powerups.value) or not (player.ssvars.powerups[powerup])) then
		active = false
	end
	
	return active
end)

rawset(_G, "P_PlayerCanGrabPowerup", function(player, pickup)

	local tbl = supersponge_powerup_info

	// Always pick up ammo or life items, regardless of current powerup.
	if (tbl[pickup].type == SS_TYPE_AMMO or tbl[pickup].type == SS_TYPE_LIFEUP) then
		return true
	end

	// Check for any "active" temporary powerups
	for i = 1, #tbl, 1 do
		if not P_IsPowerupActive(player, i) continue end // The powerup is not active, so ignore it.
		if (tbl[i].type == tbl[pickup].type and i == pickup) // Already have it.
		or (tbl[i].type == tbl[pickup].type and (tbl[i].priority != -1 and tbl[pickup].priority != -1)) then // Don't pick up another weapon / gear, unless it can be combined.
			return false
		end
	end
	
	return true
end)

// P_GiveSSPowerup
// Gives the player their respective powerups.
rawset(_G, "P_GiveSSPowerup", function(player, powerup, sfx, amount)

	local val = 1
	local tbl = supersponge_powerup_info
	local sound = tbl[powerup] and tbl[powerup].sound or sfx_sbpkup
	local type = tbl[powerup].type or SS_TYPE_NONE
	local ammo = tbl[powerup] and tbl[powerup].ammo or 0
	local ammotype = tbl[powerup].ammotype != nil and tbl[powerup].ammotype or "none"
	local maxammo = tbl[powerup].maxammo or 0
	
	if not (player and player.valid) then return false end // not valid
	
	if not (tbl[powerup]) then return false end // doesn't exist
	
	if (sfx) then
		S_StartSound(player.mo, sound, player)
	end
	
	if (ammo and ammotype != "none") then 
	
		// override ammo with set ammo.
		if (amount != nil) then
			ammo = amount
		end
	
		if (ammotype == "things") then
			// initalize things array
			if (player.ssvars.ammo and player.ssvars.ammo["things"] == nil) then
				player.ssvars.ammo["things"] = {}
			end
			// initalize subarray
			if (player.ssvars.ammo and player.ssvars.ammo["things"] and not player.ssvars.ammo["things"][powerup]) then
				player.ssvars.ammo["things"][powerup] = {}
			end
		else
			player.ssvars.ammo[ammotype] = $ + ammo		
			 // Don't try to go over the limit.
			if (player.ssvars.ammo[ammotype] > maxammo) then
				player.ssvars.ammo[ammotype] = maxammo
			end	
		end
	end
	
	if (type == SS_TYPE_LIFEUP) then P_GivePlayerLives(player, 1) return true end // Grant the player an extra life.
	
	if (type != SS_TYPE_WEAPON and type != SS_TYPE_GEAR) then return true end // Only add weapons and gear to the player's powerup array.
	
	// Set additional health for space suit.
	if (powerup == SS_POWERUP_SUIT) then
		val = 2
	end
	
	player.ssvars.powerups[powerup] = val
	
	return true
end)

// P_RemoveSSPowerups
// Removes every single powerup from Spongebob's item array
rawset(_G, "P_RemoveSSPowerups", function(player, toss, randangle)

	local tbl
	local limit
	local angle
	local ammo
	local ammotype
	
	tbl = supersponge_powerup_info

	for id = 1, #tbl, 1 do
	
		if not (tbl[id]) then continue end // doesn't exist.
		
		if not (player.ssvars.powerups[id] and player.ssvars.powerups[id] > 0) then continue end
		
		player.ssvars.powerups[id] = 0
		
		// toss the item with a set angle
		if (toss) then
		
			// randomized angle toss
			if (randangle)
				angle = FixedAngle(P_RandomRange(0, 315)*FRACUNIT)
			else
				angle = player.drawangle
			end
			
			P_DoPowerupToss(player, id, angle)
		else
			ammo = tbl[id] and tbl[id].ammo or false
			ammotype = tbl[id] and tbl[id].ammotype or "none"
			if not (ammo and ammotype and ammotype:lower() != "none") then 
				continue // no defined ammo type
			end
			// Reset any player ammo associated with the weapon.
			if (tbl[id].ammotype == "things")
				if (player.ssvars.ammo[ammotype] == nil) then
					player.ssvars.ammo[ammotype] = {}
				end
				player.ssvars.ammo[ammotype][id] = {}
			else
				player.ssvars.ammo[ammotype] = 0
			end
		end
	end
end)

rawset(_G, "P_GetCurrentSSGear", function(player)

	local count
	local gear
	
	if not (player.ssvars and player.ssvars.powerups) then
		return 0
	end
	
	gear = 0
	count = P_GetSSPowerupCount(player, SS_TYPE_GEAR)
	
	for i = 1, #supersponge_powerup_info, 1 do
		if not (supersponge_powerup_info[i] and supersponge_powerup_info[i].type) then continue end // no defined type
		if (i != SS_POWERUP_PIE and supersponge_powerup_info[i].type != SS_TYPE_GEAR) then continue end // gear only
		if not (player.ssvars.powerups[i]) then continue end // not in inventory
		// If we have already found a gear, compare it to the next item to check it's priority.
		if (gear and supersponge_powerup_info[gear].priority > supersponge_powerup_info[i].priority) then
			break
		end
		gear = i
	end
	
	return gear
end)

rawset(_G, "P_GetCurrentSSWeapon", function(player)
	local id = 0
	local count = P_GetSSPowerupCount(player, SS_TYPE_WEAPON)
	for i = 1, #supersponge_powerup_info, 1 do
		if not (supersponge_powerup_info[i] and supersponge_powerup_info[i].type) then continue end // no defined type
		if not (supersponge_powerup_info[i].type == SS_TYPE_WEAPON) then continue end // weapons only
		if (supersponge_powerup_info[i].priority == -1 and count > 1) then continue end // ignore balloon if we have more than one weapon
		if (player.ssvars.powerups[i] != nil and player.ssvars.powerups[i] > 0) then
			return i // This weapon is the current weapon.
		end
	end
	return 0
end)

local function LockonFix(mobj)
	if not (mobj.target and mobj.target.valid) then return end
	mobj.scale = mobj.target.scale // Hack to fix jittery scaling when spawning.
end
addHook("MobjThinker", LockonFix, MT_LOCKON)